/*******************************************************************************
SBFixTeleportersURL

== Description ==
This mutator runs once after a map is loaded and checks all Teleporters for an
URL that points to another level (technically: the URL contains ":" or "#").
If such a teleporter is found, it will be disabled.
Travalling teleporters are found in MH-FissionSmelter, MH-NP04Hyperion,
MH-DKdark, MH-Decks, ...

Optionally a MonsterEnd is spawned at that location with a CollisionCylinder of
same size as the teleporter.

== Contact ==
Mail: SeriousBarbie at googlemail dot com

== History ==
=== 2018-10-22 ===
* the Teleporter's property "bEnabled" and "Tag" is copied to the MonsterEnd's
property "bInitiallyActive" and "Tag"
* renamed project from "SBMutFixTeleportersURL" to "SBFixTeleportersURL"
=== 2016-12-12 ===
* Because some editions of MonsterHunt do a "static" replacement of MonsterEnd's
in their PostBeginPlay() code, later added MonsterEnds are not treated. So I
moved the fixing code to the PreBeginPlay() event of this mutator.
* added condition "InStr(T.URL, ":") >= 0" for detecting travelling URLs
* added hack to prevent a spawned MonsterEnd from the CheckReplacement chain
=== 2016-08-21 ===
* first release
*******************************************************************************/

class SBFixTeleportersURL expands Mutator config(SBFixTeleportersURL);

#exec OBJ LOAD File="..\System\Monsterhunt.u"

var config bool	bActive;
var config bool	bLogActions;
var config bool	bSpawnMHEnd;
var config bool	bMHEndVisible;



function bool FixTeleporterWithURL(Teleporter T) {
/*******************************************************************************
Returns TRUE if *T* was disabled, because it has a travelling URL. A MonsterEnd
is also spawned at *T*'s location.
If *bForce*, the teleporter is treated as if it has a travelling URL.

If URL contains the map itself, the teleporter is treated also.
*******************************************************************************/
local MonsterEnd MHEnd;
local string s;
local bool bGameRelevantDefault;

	if ( ! IsTravellingURL(T.URL))
		Return false;

	if (bLogActions)
	{
		if (bSpawnMHEnd)
		{
			s = " and spawning a";
			if (bMHEndVisible)
				s = s @ "visible";
			else
				s = s @ "hidden";
			s = s @ "MonsterEnd";
		}
		Log(self @ "FixTeleporterWithURL: " @ T @ "has travelling URL '" $ T.URL $ "', disabling teleporter" $ s);
	}
	T.URL = "";
	T.SetCollision(False, False, False);// don't replace to avoid error messages "destination not found" ingame
	T.bHidden = True;
	T.Disable('Touch');
	if (bSpawnMHEnd) // put a MHEnd also at the same place:
	{
		bGameRelevantDefault = class'Monsterhunt.MonsterEnd'.Default.bGameRelevant;
		class'Monsterhunt.MonsterEnd'.Default.bGameRelevant = true; // avoid passing to CheckReplacement()
		MHEnd = Spawn(class'Monsterhunt.MonsterEnd', , T.Tag, T.location);
		class'Monsterhunt.MonsterEnd'.Default.bGameRelevant = bGameRelevantDefault;
		if (MHEnd != None)
		{
			MHEnd.bGameRelevant = bGameRelevantDefault;
			MHEnd.SetCollisionSize(FMax(t.CollisionRadius, MHEnd.CollisionRadius), FMax(t.CollisionHeight, MHEnd.CollisionHeight));
			MHEnd.bInitiallyActive = T.bEnabled;
			if ( ! Level.Game.IsRelevant(MHEnd)) // now pass the Actor to the CheckReplacement() chain
				MHEnd.Destroy();
		}
	}
	Return True;
}



function int FixTeleportersWithURL() {
/*******************************************************************************
Disables all Teleporters with a travelling URL and spawns a MHEnd there.
*******************************************************************************/
local Teleporter T;
local int result;
local bool bMonsterEndDefaultbHidden;

	if (bSpawnMHEnd)
	{
		bMonsterEndDefaultbHidden = class'Monsterhunt.MonsterEnd'.default.bHidden;
		class'Monsterhunt.MonsterEnd'.default.bHidden = ! bMHEndVisible;
	}

	foreach AllActors(class 'Teleporter', T)
		if (FixTeleporterWithURL(T))
			result++;

	// reset default value:
	if (bSpawnMHEnd)
		class'Monsterhunt.MonsterEnd'.default.bHidden = bMonsterEndDefaultbHidden;

	return result;
}



function bool IsTravellingURL(string URL) {
/*******************************************************************************
Returns TRUE if *URL* contains '#' or ':'
*******************************************************************************/

	if (InStr(URL, "#") >= 0)
		Return true;

	if (InStr(URL, ":") >= 0)
		Return true;

	Return false;
}



event PreBeginPlay() {
	super.PreBeginPlay();
	if(bActive)
		Log(self $ ".PreBeginPlay: number of fixed Teleporters with URL:" @ FixTeleportersWithURL());
	else
		Log(self $ ".PreBeginPlay: Mutator via INI settings disabled, doing nothing");

}



defaultproperties {
	bActive=true
	bLogActions=true
	bSpawnMHEnd=true
	bMHEndVisible=true
}
